(* $Id: godi_file.mli 495 2004-09-30 21:38:03Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

(* Advanced file operations *)

val read_dir : string -> string list
  (* Read directory including "." and ".." *)

val file_is_dir : string -> bool
  (* Whether the file exists and is a directory *)

val file_exists : ?symlink:bool -> string -> bool
  (* Whether the file exists.
   *
   * @param symlink If true, it is acceptable that the file is a symlink
   * that points to a non-existing location
   *)

val file_type : ?is_regular:bool ->
                ?is_dir:bool ->
		?is_existing:bool ->
                ?is_symlink:bool ->
		?is_not_symlink:bool -> string -> bool
  (* Checks the file type. The function returns [true] if all of the
   * enabled checks yield a positive result:
   * - [is_regular]: The file exists and is a regular file, or a
   *   symbolic link to an existing regular file
   * - [is_dir]: The file exists is a directory, or a symbolic link
   *   to an existing regular file
   * - [is_existing]: The file exists, or is a symbolic link to
   *   an existing file
   * - [is_symlink]: The file exists and is a symbolic link, no matter
   *   to which location it points to
   * - [is_not_symlink]: The file exists and is not a symbolic link
   *)

val file_mtime : string -> float
  (* Returns the time of last modification (or Sys_error) *)

val remove : ?recurse:bool -> string -> unit
  (* Removes the named file. If [recurs], the removal is recursive,
   * and it is no error when the file is not an existing pathname.
   * [Sys_error] when the removal fails.
   *)

val remove_dir : ?recurse:bool -> string -> unit
  (* Removes the named directory. If [recurse], the removal is recursive,
   * and it is no error when the directory does not exist.
   * [Sys_error] when the removal fails.
   *)

val create_dir : ?recurse:bool -> string -> unit
  (* Creates the named directory. If [recurse], it is tried to create the
   * missing parent directories, too, and it is no error when the directory
   * already existed before.
   * [Sys_error] when the creation fails.
   *)

val create_tmp_dir : unit -> string
  (* Creates a new temporary directory *)

val copy : ?prefer_link:bool -> ?remove:bool -> string -> string -> unit
  (* Copies the first file to the second. By default, the second file is
   * overwritten.
   *
   * The permission bits are copied land 0o777. The owner information is
   * not copied, neither are the file times.
   *
   * @param prefer_link When possible, the second file becomes another link
   * of the first file.
   *
   * @param remove The second file is not overwritten, but deleted if existing,
   * and again created.
   *)

type file_id

val file_id : string -> file_id
  (* Returns an identifier for the file. It is required that the file name
   * f1 and f2 refer to the same file when file_id f1 = file_id f2; however,
   * the reverse need not be true.
   *
   * A trivial implementation is to return a new number each time the
   * function is called. However, this is very bad.
   *
   * For Unix, a file_id is the pair of filesystem device and inode number.
   *)

val same_file : string -> string -> bool
  (* Returns [true] when both files exist and both names refer to the same
   * file. It is ok to return [false] when it cannot be determined whether
   * the files are the same.
   *)

type file_fingerprint

val file_fingerprint : string -> file_fingerprint
  (* When the fingerprint of the file changes, this is a hint that the
   * contents of the file have changed.
   *
   * For Unix, the fingerprint contains mtime, ctime, and file size.
   *)

type tmpdir
  (* A registered temporary directory *)

val tmpdir_path : tmpdir -> string
  (* Returns the absolute path to the temporary directory *)

val allocate_tmpdir : unit -> tmpdir
  (* Allocates a new temporary directory *)

val release_tmpdir : tmpdir -> unit
  (* Releases the temporary directory (i.e. deletes it) *)

val release_all_tmpdirs : unit -> unit
  (* Releases all known temporary directories *)


val execute : ?setx:bool -> string -> unit
  (* Execute the shell commands in the string with the "exec" system call.
   * On success, this function does not return.
   *
   * Note: This is only needed for self-updating packages, and not during
   * the bootstrap.
   *
   * setx: whether to pass the -x option to the shell. Default: false
   *)
